#include <CapacitiveSensor.h>

//////////////// INPUTS AND VARIABLES DEFINITION ////////////////

boolean Break = false;

// Define Reactions thresholds
  //Flirty 
    int thresholdFlirt = 30;
    int pitchFlirt = 800; int screamTimeFlirt = 50;
  //Soft touch
    int thresholdSoft = 45;
    int pitchSoft = 400; int screamTimeSoft = 100;
    int fleeTimeSoft = 150; int minDistanceSoft = 20;
  //Intense touch
    int thresholdIntense = 60;
    int pitchIntense = 800; int screamTimeIntense = 500;
    int fleeTimeIntense = 400; int minDistanceIntense = 25;
  //Too much touch
    int pitchTooMuch = 3000; int screamTimeTooMuch = 1000;
    int fleeTimeTooMuch = 700; int minDistanceTooMuch = 35;


//Create Touch variables 
  long touchValue;
  int touchLow = 1023;
  int touchHigh = 0;

//Ultrasonic sensor setup
  int trigPin = 6;
  int echoPin = 11;
  double duration = 0;
  double distance = 0; 

//Define DC Motors 
  // Motor A connections
    int enA = 9;
    int in1 = 8;
    int in2 = 7;
  // Motor B connections
    int enB = 3;
    int in3 = 5;
    int in4 = 4;

//Define capacitive Sensor 
  CapacitiveSensor capSensor = CapacitiveSensor(13, 12);
  


//////////////// VOID SET-UP ////////////////

void setup() {

Serial.begin(9600);

//Ultrasonic sensor setup
    pinMode(trigPin, OUTPUT);
    pinMode(echoPin, INPUT);
  
 // Set all the motor control pins to outputs
    pinMode(enA, OUTPUT);
    pinMode(enB, OUTPUT);
    pinMode(in1, OUTPUT);
    pinMode(in2, OUTPUT);
    pinMode(in3, OUTPUT);
    pinMode(in4, OUTPUT);
  
// Turn off motors - Initial state
    digitalWrite(in1, LOW);
    digitalWrite(in2, LOW);
    digitalWrite(in3, LOW);
    digitalWrite(in4, LOW);

//5 first seconds after reset are the calibration of the touch sensing. During those 5 seconds, touch the plant as much as possible, and don't touch it at all, to get the range of values to remap. 
CalibrateTouchValues();
Serial.println(touchHigh); Serial.println("touch high");
Serial.println(touchLow); Serial.println("touch low");
Serial.println("............");

}


//////////////// FUNCTIONS ////////////////

///////Touch calibration 

void CalibrateTouchValues() {
  //Sound to indicate the start of the program
    tone(2, 800, 100),
    delay(5);
  
  while(millis() < 5000) {
  long touchValue = capSensor.capacitiveSensor(30);
  if (touchValue > touchHigh) {
    touchHigh = touchValue;
  }
  if (touchValue < touchLow) {
    touchLow = touchValue;
  }
  }
  //Calibration is done
    tone(2, 800, 100),
    delay(5);
}


///////Find obstacles

  void getObstacleDistance() {
    //Get duration between sensor and something
      digitalWrite(trigPin,LOW);
      delayMicroseconds(100);
      digitalWrite(trigPin,HIGH);
      delayMicroseconds(1000);
      digitalWrite(trigPin,LOW);
      duration = pulseIn( echoPin,HIGH);

    //Calculate distance between sensor and something
      if(duration >0){
      duration = duration/2;
      distance = duration*340*100/1000000;
      Serial.println(distance);
      }
      return distance;
  }


///////Motor moves functions

 void moveForward(int Time) {
    // Turn on motors
      digitalWrite(in1, HIGH);
      digitalWrite(in2, LOW);
      digitalWrite(in3, HIGH);
      digitalWrite(in4, LOW);

    // Accelerate from zero to max speed
      for (int i = 100; i < 255; i++) {
        analogWrite(enA, i);
        analogWrite(enB, i);
        delay(5);
        }
      delay(Time);
      moveStop();

 }

 void turnLeft() {
    moveForward(5);
    digitalWrite(in1, HIGH);
    digitalWrite(in2, LOW);
    digitalWrite(in3, LOW);
    digitalWrite(in4, HIGH);
    delay(350);
    moveStop();
    }

 void moveStop() {
    digitalWrite(in1, LOW);
    digitalWrite(in2, LOW);
    digitalWrite(in3, LOW);
    digitalWrite(in4, LOW);
  }



//////////Flee movement loop 
void FleeLoop(int fleeTime, int minDistance) {
  if (Break == false) {

   getObstacleDistance();

   if (distance > minDistance) {
    moveForward(fleeTime);
    Break = true;
   }

   else {
    turnLeft();
    getObstacleDistance();

      if (distance > minDistance) {
        moveForward(fleeTime);
        Break = true; 
      }
  
      else {
        moveStop();
      }
    }
   }
   Break = false;
}




//////////////// VOID LOOP ////////////////

void loop() {
  
  //Read and remap capacitive sensor value
    long touchValue = capSensor.capacitiveSensor(30);
    //Serial.println(touchValue); Serial.println("touch value");
    
    int touch = map(touchValue, touchLow, touchHigh, 0, 100);
    //Serial.println(touch); Serial.println("touch NORM");
    //Serial.println("................");
    delay(500);
    

  if (touch > 23 && touch < thresholdFlirt) {
    //Sound
      tone(2, pitchFlirt, screamTimeFlirt);
      delay(10);
   }
  
  
  if (touch > thresholdFlirt && touch < thresholdSoft) {
    //Sound
      tone(2, pitchSoft, screamTimeSoft);
      delay(10);
      
    //Move 
      FleeLoop(fleeTimeSoft, minDistanceSoft);
    }
    

  if (touch > thresholdSoft && touch < thresholdIntense) {
    //Sound
        tone(2, pitchIntense, screamTimeIntense);
        delay(10);

    //Move 
      FleeLoop(fleeTimeIntense, minDistanceIntense);
    }
    

  if (touch > thresholdIntense ) {
    //Sound
        tone(2, pitchTooMuch, screamTimeTooMuch);
        delay(10);

    //Move 
      FleeLoop(fleeTimeTooMuch, minDistanceTooMuch);

    
    }


}
  
